﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TrolleybusProject.Entities;

namespace TrolleybusProject.Drawnings;

public class DrawningBus
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityBus? EntityBus { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки троллейбуса
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки троллейбуса
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки троллейбуса
    /// </summary>
    public readonly int _drawningBusWidth = 150;
    /// <summary>
    /// Высота прорисовки троллейбуса
    /// </summary>
    public readonly int _drawningBusHeight = 86;
    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;
    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningBusWidth;
    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningBusHeight;



    /// <summary>
    /// Конструктор пустой
    /// </summary>
    private DrawningBus()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }




    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
 
    public DrawningBus(int speed, double weight, Color bodyColor):this()
    {
        EntityBus = new EntityBus(speed, weight, bodyColor);

    }


    /// <summary>
    /// Конструктор для наследников 
    /// </summary>
    /// <param name="drawningBusWidth">Ширина прорисовки автобуса</param>
    /// <param name="drawningBusHeight">Высота прорисовки автобуса</param>


    protected DrawningBus(int drawningBusWidth, int drawningBusHeight) : this()
    {
        _drawningBusWidth= drawningBusWidth;
        _drawningBusHeight = drawningBusHeight;

    }






    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (width > _drawningBusWidth || height > _drawningBusHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;



            if (_startPosY.HasValue && _startPosX.HasValue)
            {
                if (_startPosX + _drawningBusWidth > width)
                {

                    _startPosX = width - _drawningBusWidth;


                }
                if (_startPosY + _drawningBusHeight > height)
                {

                    _startPosY = height - _drawningBusHeight;


                }
                if (_startPosX < 0)
                {
                    _startPosX = 0;
                }
                if (_startPosY < 0)
                {
                    _startPosY = 0;
                }



            }



            return true;
        }






        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }


        _startPosX = x;
        _startPosY = y;



        if (_startPosX < 0)
        { _startPosX = 0; }
        if (_startPosY < 0)
        { _startPosY = 0; }

        if (x + _drawningBusWidth > _pictureWidth)
        {

            _startPosX = _pictureWidth - _drawningBusWidth;


        }
        if (y + _drawningBusHeight > _pictureHeight)
        {

            _startPosY = _pictureHeight - _drawningBusHeight;


        }


    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityBus == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityBus.Step > 0)
                {
                    _startPosX -= (int)EntityBus.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityBus.Step > 0)
                {
                    _startPosY -= (int)EntityBus.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityBus.Step < _pictureWidth - _drawningBusWidth)
                {
                    _startPosX += (int)EntityBus.Step;
                }
                return true;

            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityBus.Step < _pictureHeight - _drawningBusHeight)
                {
                    _startPosY += (int)EntityBus.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityBus == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);


        Brush brushBodyColor = new SolidBrush(EntityBus.BodyColor);

        //троллейбус границы
        g.DrawEllipse(pen, _startPosX.Value + 22, _startPosY.Value +
            42, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 104, _startPosY.Value +
           42, 20, 20);


        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value +
          8, 140, 41);


        //троллейбус
        g.FillRectangle(brushBodyColor, _startPosX.Value + 5, _startPosY.Value +
        8, 140, 41);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value +
        8, 140, 41);




        //дверь
        g.DrawRectangle(pen, _startPosX.Value + 52, _startPosY.Value +
          18, 24, 31);


        //окна

        Brush brBlue = new SolidBrush(Color.LightBlue);

        g.FillEllipse(brBlue, _startPosX.Value + 8, _startPosY.Value +
         12, 15, 20);

        g.FillEllipse(brBlue, _startPosX.Value + 33, _startPosY.Value +
          12, 15, 20);


        g.FillEllipse(brBlue, _startPosX.Value + 80, _startPosY.Value +
         12, 15, 20);
        g.FillEllipse(brBlue, _startPosX.Value + 97, _startPosY.Value +
          12, 15, 20);
        g.FillEllipse(brBlue, _startPosX.Value + 113, _startPosY.Value +
          12, 15, 20);
        g.FillEllipse(brBlue, _startPosX.Value + 129, _startPosY.Value +
         12, 15, 20);









        g.DrawEllipse(pen, _startPosX.Value + 8, _startPosY.Value +
          12, 15, 20);

        g.DrawEllipse(pen, _startPosX.Value + 33, _startPosY.Value +
          12, 15, 20);


        g.DrawEllipse(pen, _startPosX.Value + 80, _startPosY.Value +
        12, 15, 20);
        g.DrawEllipse(pen, _startPosX.Value + 97, _startPosY.Value +
         12, 15, 20);
        g.DrawEllipse(pen, _startPosX.Value + 113, _startPosY.Value +
          12, 15, 20);
        g.DrawEllipse(pen, _startPosX.Value + 129, _startPosY.Value +
         12, 15, 20);
















    }
}
