﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TrolleybusProject.Drawnings;
using TrolleybusProject.MovementStrategy;

namespace TrolleybusProject;

public partial class FormTrolleybus : Form
{
    private DrawningBus? _drawningBus;

    private AbstractractStrategy? _strategy;
   

    public DrawningBus SetBus
    {
        set
        {
            _drawningBus = value;
            _drawningBus.SetPictureSize(pictureBoxTrolleybus.Width,
            pictureBoxTrolleybus.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    public FormTrolleybus()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки машины
    /// </summary>
    private void Draw()
    {
        if (_drawningBus == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxTrolleybus.Width,
        pictureBoxTrolleybus.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningBus.DrawTransport(gr);
        pictureBoxTrolleybus.Image = bmp;
    }




    private void buttonMove_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningBus.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningBus.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningBus.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningBus.MoveTransport(DirectionType.Right);
                break;

        }
        if (result)
        {
            Draw();


        }

    }






  
 




    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }
        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableBus(_drawningBus),
            pictureBoxTrolleybus.Width, pictureBoxTrolleybus.Height);
        }
        if (_strategy == null)
        {
            return;
        }
        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();
        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }

}
